# This file was automatically generated. DO NOT EDIT.
# If you have any remark or suggestion do not hesitate to open an issue.
from typing import List

from .types import (
    HubStatus,
)


HUB_TRANSIENT_STATUSES: List[HubStatus] = [
    HubStatus.ENABLING,
    HubStatus.DISABLING,
]
"""
Lists transient statutes of the enum :class:`HubStatus <HubStatus>`.
"""
